@@ -20,26 +20,34 @@ pub fn fmt<W: Write>(
         writeln!(w, "// {} {}", qtype.unwrap(), qname.unwrap())?;
     }
 
+    writeln!(w, "#[rustfmt::skip]")?;
     writeln!(w, "const {}: [u8; {}] = [", name, msg.len())?;
 
     let chunks = msg.chunks(CHUNK_LEN);
     let mut max_chunk_len = 0;
 
-    for chunk in chunks {
+    for (index, chunk) in chunks.enumerate() {
         let len = chunk.len();
         max_chunk_len = max_chunk_len.max(len);
 
-        write!(w, "    ")?; // indentation
-
         //
         // chunk bytes as hex literals
         //
-        for b in chunk {
-            write!(w, "{:#04x?}, ", b)?;
+        for (i, b) in chunk.iter().enumerate() {
+            let pfx = if i == 0 { "    " } else { " " };
+            write!(w, "{}{:#04x?},", pfx, b)?;
         }
+
+        //
         // fill the last, possibly shorter, line
-        for _ in 0..(max_chunk_len - len) {
-            write!(w, "      ")?;
+        //
+        if len < max_chunk_len {
+            write!(w, " /*")?;
+            let n_spaces = (max_chunk_len - len) * 6 - 4 - 1;
+            for _ in 0..n_spaces {
+                write!(w, " ")?;
+            }
+            write!(w, "*/")?;
         }
 
         //
@@ -55,7 +63,7 @@ pub fn fmt<W: Write>(
             write!(w, "{}", c)?;
         }
 
-        writeln!(w, "|")?;
+        writeln!(w, "| {}", (index * CHUNK_LEN))?;
     }
 
     writeln!(w, "];")?;
