@@ -29,6 +29,10 @@ pub {{ as }} fn main() -> Result<()> {
 
     let args = Args::get()?;
     let mut format = Format::new(&args);
+    if args.has_read_path() {
+        return format.read();
+    }
+
     let mut client = Client::new(args.config.clone()){{ aw }}?;
 
     for qname in args.qnames.iter() {
@@ -36,7 +40,8 @@ pub {{ as }} fn main() -> Result<()> {
         let size = client
             .query_raw(qname, args.qtype(), Class::In, &mut buf){{ aw }}?;
         let elapsed = now.elapsed().expect("time failed");
-        format.add(qname, &buf[..size], now, elapsed)?;
+        format.add(Some(qname), Some(args.qtype()), &buf[..size], Some(args.config.nameserver()),
+                   Some(now), Some(elapsed))?;
     }
 
     format.done()?;
